@@ -1,7 +1,8 @@
 import inspect
+import logging
+
 import numba as nb
 import numpy as np
-import copy
 from numba.experimental.jitclass.base import JitClassType
 from numba.experimental import jitclass
 from typing import Optional
@@ -9,6 +10,7 @@ from abc import ABC
 
 from .solve_states import SolveEvent
 
+logger = logging.getLogger(__name__)
 
 class Solution:
     """Default class which holds the solution in arrays.
@@ -57,7 +59,7 @@ class Solution:
         self.__init__()
 
 
-class Interface:
+class Interface:  # pragma no cover
     """
     The interface is used to connect a Model to the solver. To solve an ODE, as a minimum the methods
     get_deriv and get_states.
@@ -182,16 +184,16 @@ class Interface:
         """
         return y
 
-    def get_next_time_event(self, t) -> Optional[tuple[int, float]]:
+    def get_next_time_event(self, t) -> Optional[tuple[list[int], float]]:
         """
         Function that is called after each converged solver step. Returns a tuple which contains an index of the next
         time event action function to be trigged, and the value of the time-step when the function is triggered.
 
         :param t: current time
         :type t: float
-        :return: Optional: tuple of index of event function and next time. If no function is triggered, it should return
-            None.
-        :rtype: Optional[tuple[int, float]]
+        :return: Optional: tuple of list of indexes of event function(s) and next time. If no function is triggered,
+                 it should return None.
+        :rtype: Optional[tuple[list[int], float]]
         """
         pass
 
@@ -299,25 +301,17 @@ class DefaultEventHandler(EventHandler):
 
 
 class _JitHelper:
+    """
+    The _JitHelper object class is used to wrap the 
+    """
     def __init__(self, model_class, *args, **kwargs):
         self.model_class = model_class
         self.args = args
         self.kwargs = kwargs
         self.python_model = None
 
-        args_ = []
-        kwargs_ = {}
-        for arg in args:
-            if isinstance(arg, _JitHelper):
-                args_.append(arg.python_model)
-            else:
-                args_.append(arg)
-        for k, v in kwargs.items():
-            if isinstance(v, _JitHelper):
-                kwargs_.update({k: v.python_model})
-            else:
-                kwargs_.update({k: v})
-
+        args_ = self._add_args(args)
+        kwargs_ = self._add_kwargs(kwargs)
 
         self.python_model = model_class(*args_, **kwargs_)
         self.compiled_model = None
@@ -329,9 +323,47 @@ class _JitHelper:
                 continue
             setattr(self, fun, getattr(self.python_model, fun))
 
+    def _add_args(self, args_: iter, jit=False):
+        args__ = []
+        for arg in args_:
+            if not self._is_reserved_nested_type(arg):
+                try:
+                    _ = iter(arg)
+                    args__.append(self._add_args(arg, jit))
+                except TypeError as e:
+                    if not "is not iterable" in e.__repr__():
+                        raise
+                    if isinstance(arg, _JitHelper):
+                        args__.append(arg(jit=jit))
+                        #args__.append(arg.python_model)
+                    else:
+                        args__.append(arg)
+            else:
+                args__.append(arg)
+        return args__
+
+    def _add_kwargs(self, kwargs_: dict, jit=False):
+        kwargs__ = {}
+        for k,v in kwargs_.items():
+            if not self._is_reserved_nested_type(v):
+                try:
+                    _ = iter(v)
+                    kwargs__.update({k: self._add_args(v, jit)})
+                except TypeError as e:
+                    if not "is not iterable" in e.__repr__():
+                        raise
+                    if isinstance(v, _JitHelper):
+                        kwargs__.update({k: v(jit=jit)})
+                        #kwargs__.update({k: v.python_model})
+                    else:
+                        kwargs__.update({k: v})
+            else:
+                kwargs__.update({k: v})
+        return kwargs__
+
     def __call__(self, jit=False):
-        """Call to JitHelper if model needs to be jitted. In arguments contain other JitHelper items, they are jitted first
-        before being passed to the jithelper function
+        """Call to _JitHelper if model needs to be jitted. In arguments contain other _JitHelper items, they are jitted 
+        first before being passed to the _JitHelper function.
 
         :param jit: True if class needs to be jitted (must obey Numba specs), or false if using regular python
         :return: Jitted (or not) model class
@@ -339,28 +371,18 @@ class _JitHelper:
         if hasattr(self.python_model, '_numba_model_'):
             return self.python_model
 
-        newargs = []
-        for arg in self.args:
-            newarg = arg
-            if isinstance(arg, _JitHelper):
-                newarg = arg(jit=jit)
-            newargs.append(newarg)
-        args = tuple(newargs)
-
-        newkwargs = {}
-        for k, v in self.kwargs.items():
-            newkwarg = {k: v}
-            if type(v) == _JitHelper:
-                newkwarg = {k: v(jit=jit)}
-            newkwargs.update(newkwarg)
-        kwargs = newkwargs
+        args = tuple(self._add_args(self.args, jit=jit))
+        kwargs = self._add_kwargs(self.kwargs, jit=jit)        
 
         compiled_model = self.jithelper(self.model_class, jit=jit, return_wrapper=False)(*args, **kwargs)
         self.compiled_model = compiled_model
         return self.compiled_model
 
+    def _is_reserved_nested_type(self, arg) -> bool:
+        return isinstance(arg, (str, np.ndarray))
+
     @staticmethod
-    def jithelper(model_class: callable, jit=True, pymodel=None, return_wrapper=False) -> callable:
+    def jithelper(model_class: callable, jit=True, return_wrapper=False) -> callable:
         """Method to create a jitted class using the spec from the model arguments. If jit=True, then all parameters
         must be resolved to numba classes.
 
@@ -368,19 +390,12 @@ class _JitHelper:
         :type model_class: :class:`solver.interface.Model`
         :param jit: To jit or not
         :type jit: bool
-        :param pymodel: An optional instance of the python model
-        :type pymodel: :class:`solver.interface.Model`
-        :param return_wrapper: To return the wrapped class, or to return the jitted class
-        :type return_wrapper: bool
         :return: Returns wrapper, jitted class, or python model (jit=False)
         :rtype: Union[:class:`numba.core.registry.CPUDispatcher`, :class:`solver.interface.Model`\
         :class:`numba.experimental.jitclass.boxing.Wrapper`]
         """
         def wrapper(*args, **kwargs):
-            if not pymodel:
-                python_model = model_class(*args, **kwargs)
-            else:
-                python_model = pymodel
+            python_model = model_class(*args, **kwargs)
 
             if not jit:
                 if return_wrapper:
@@ -400,36 +415,35 @@ class _JitHelper:
                 else:
                     spec.append((v, nbtype))
 
+
             @jitclass(spec=spec)
             class Wrapper(model_class):
                 pass
 
-            if return_wrapper:
-                return Wrapper
-            else:
-                return Wrapper(*args, **kwargs)
+            return Wrapper(*args, **kwargs)
 
         return wrapper
 
-    def reset(self):
+    def _reset(self):
         """
-        Recursive method to reset all numba compiled jitclasses to their default values. Uses the python model to
-        determine the initial values of all properties. If the property is a _JitHelper type, then use recursion.
-
-        :return:
+        Method wrapper to reset numba compiled jitclasses to their default values. Calls the reset() method on the model
         """
         if not self.compiled_model:
             raise AttributeError('cannot reset - model not compiled')
 
-        for k, v in self.python_model.__dict__.items():
-            if isinstance(v, _JitHelper):
-                v.reset()
-            setattr(self.compiled_model, k, getattr(v, 'compiled_model', v))
+        reset = getattr(self.compiled_model, 'reset', None)
+        if not reset:
+            logger.warning("could not reset model - no reset method specified")
+            return
 
+        reset()
 
 class Model:
     """Helper class to decorate models.
-    Holds the python model, interface and compiled model (generated once solver is compiled).
+    Holds the python model, interface and compiled model (generated once solver is compiled). The model allows
+    inputs in its constructor that can be compiled with numba, for example: homogeneous lists, strings, floats.
+    When the solver is called, the solver attempts to jit the model. If it fails, it will raise a numba error.
+    If the user wishes, they may revert to jit=False.
     """
 
     def __init__(self, model_class: object, interface: str | Interface | None = None, iscomponent: bool = False):
@@ -439,21 +453,27 @@ class Model:
         self.python_model = None
         self.iscomponent = iscomponent
         self.compiled_model = None
+        self._JitHelper = None
 
     def __call__(self, *args, **kwargs):
-        """When calling the Model object, the model is jitted. If the model is a component it returns the jitted class
+        """When calling the Model object, the model is jitted. If the model is a component it returns the jitted class.
+        Creates a new class of type Model, the calls its init function.
 
         :param args: Arguments to the Model
         :param kwargs: Keyword arguments to the Model
-        :return: returns either the Model class ifself, or a jitted/non-jitted model_class
+        :return: returns either a new instance of the Model class ifself, or a new instance of the jitted/non-jitted
+        model_class
         :rtype: :class:`solver.interface.Model` | :class:`numba.core.registry.CPUDispatcher` | :class:object
         """
-        self.jithelper = _JitHelper(self.model_class, *args, **kwargs)
-        self.python_model = self.jithelper.python_model
 
-        if self.iscomponent:
-            return self.jithelper
-        return self
+        new_class = self.__new__(self.__class__)
+        new_class.__init__(self.model_class, self.interface, self.iscomponent)
+        new_class._JitHelper = _JitHelper(new_class.model_class, *args, **kwargs)
+        new_class.python_model = new_class._JitHelper.python_model
+        if new_class.iscomponent:
+            return new_class._JitHelper
+
+        return new_class
 
     @classmethod
     def with_interface(cls, interface: type(Interface) | str | None = None):
@@ -512,7 +532,7 @@ class Model:
             return np.array([0], dtype='float')
 
         def get_next_time_event_mock(self, t):
-            return -1, -1
+            return [-1], -1
 
         if not has_events:
             interface.get_event_results = get_event_results_mock
@@ -529,8 +549,9 @@ class Model:
         :return: Returns the generated model interface
         :rtype: :class:`solver.interface.Interface`
         """
+        logger.info("compiling interface")
         interface = self.load_interface()
-        model_ = self.jithelper(jit=jit)
+        model_ = self._JitHelper(jit=jit)
         self.interface = _JitHelper.jithelper(interface, jit)(model_)
         self._interface_dict = interface(model_).__dict__  # Instantiate interface to get its default values
         self.compiled_model = self.interface.model
@@ -540,7 +561,8 @@ class Model:
     def reset(self):
         """
         Resets the model and its interface. Can be called directly, or indirectly from \
-        :meth:`solver.numerous_solver.NumerousSolver.reset`. Set all values to their initial ones (including states)
+        :meth:`solver.numerous_solver.NumerousSolver.reset`. Calls the compiled model, and it's reset method, which must
+        be specified by the user.
 
         Raises an error if interface is not compiled
 
@@ -549,15 +571,9 @@ class Model:
         if not self.interface:
             raise AttributeError("Interface not compiled - cannot reset")
 
-        self.jithelper.reset()
+        self._JitHelper._reset()
 
         for k,v in self._interface_dict.items():
             if k == 'model':
                 continue
             setattr(self.interface, k, v)
-
-
-
-
-
-
