@@ -81,10 +81,6 @@ class NumerousSolver:
 
         self.number_of_events = len(self.g)
 
-        self.has_time_events = False
-        if self.interface.get_next_time_event(0) is not None:
-            self.has_time_events = True
-
         self.interface.set_states(y0)
 
         self.info = None
@@ -179,8 +175,8 @@ class NumerousSolver:
             t_event = t
             y_event = y
             t_event_previous = -1
-            time_event_ix, t_next_time_event = interface.get_next_time_event(t)
-            if time_event_ix > 0:
+            time_event_ixs, t_next_time_event = interface.get_next_time_event(t)
+            if time_event_ixs[0] >= 0:
                 te_array[2] = t_next_time_event
             else:
                 te_array[2] = max(te_array)
@@ -272,8 +268,8 @@ class NumerousSolver:
                     return status, t_r, sol(t_r, t, y)
 
                 if step_converged:
-                    time_event_ix, t_next_time_event = interface.get_next_time_event(t)
-                    if time_event_ix > 0:
+                    time_event_ixs, t_next_time_event = interface.get_next_time_event(t)
+                    if time_event_ixs[0] >= 0:
                         te_array[2] = t_next_time_event
                     else:
                         te_array[2] = max(te_array)
@@ -303,7 +299,8 @@ class NumerousSolver:
                     t_previous = t
                     if abs(t - t_next_time_event) < 1e-6:
                         interface.set_states(y)
-                        y_previous = interface.run_time_event_action(t, y, time_event_ix)  # todo: use interface to get y.
+                        for time_event_ix in time_event_ixs:
+                            y_previous = interface.run_time_event_action(t, y_previous, time_event_ix)  # todo: use interface to get y.
                         interface.set_states(y_previous)
                         solve_event_id = interface.post_time_event(t, y_previous)
                         if solve_event_id != SolveEvent.NoneEvent:
